//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Program      : B2DS                                                 +
//+ Module       : B2DS.C                                               +
//+ Purpose      : Extract user from the bindery to a UIMPORT file      +
//+ Date         : 15apr93                                              +
//+ Written by   : Bernd Kunze                                          +
//+ Usage        : B2DS -? will go for help                             +
//+ Remarks      : v1.00 * 15apr93 * Initial version                    +
//+              :                   Including source code              +
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include	<stdio.h>
#include	<conio.h>
#include	<ctype.h>
#include	<string.h>
#include    <stdlib.h>

#include    <nwalias.h>
#include    <nwbindry.h>
#include    <nwcaldef.h>
#include    <nwconnec.h>
#include    <nwdsdefs.h>
#include    <nwmisc.h>
#include    <nwserver.h>

#ifndef SUCCESSFUL
#define     SUCCESSFUL      0
#endif

int			ArgCount;
int			LoginScript = FALSE;
int			TestMode = FALSE;
int         ServerSpecified = FALSE;
int         OutFileSpecified = FALSE;
int         CreateCtrlFile = FALSE;
int         CtrlFileSpecified = FALSE;
int         LineCount = 0;

FILE        *UData;
BYTE        PropertyValue [128];

NWFLAGS NWFAR MoreFlag;
NWFLAGS NWFAR PropertyFlags;


char		SourceServer [48];					// Name of the server
char        OutputFile [_MAX_PATH];

char        ControlFile [_MAX_PATH];
char        Context [MAX_DN_CHARS];
char        Dummy;

char        CreateHomeDir;
char        HomeDirBase [_MAX_PATH];
char        HomeDirVolume [MAX_DN_CHARS];
char        ServerName [MAX_DN_CHARS];

FILE  *CFile;

NWCONN_HANDLE NWFAR *TargetConnectionID;        // The connection ID for
                                                // bindery search
unsigned int  CCode;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Bindery search variables.                                           +
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

DWORD NWFAR  ObjectID;
char  NWFAR  ObjectName [48];
WORD  NWFAR  ObjectType;
BYTE  NWFAR  ObjectPropertiesFlag;
BYTE  NWFAR  ObjectFlags;
BYTE  NWFAR  ObjectSecurity;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Prototypes                                                          +
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void   	    Usage (void);
void        SetupSourceServer (void);
void        BrowseBindery (void);
void        CreateControlFile (void);
void        IncLine (void);
void        PressAnyKey (void);
int         GetPropertyValue (char NWFAR *, NWOBJ_TYPE, char *);

void main (int argc, char *argv [])
{
	printf ("B2DS v1.00 * Bindery-to-NDS conversion tool\n");

    CCode = NWCallsInit (NULL, NULL);
    if (CCode != SUCCESSFUL)
    {
        exit (0);
    }
	for (ArgCount = 1 ; ArgCount < argc ; ArgCount++)
	{
		if (argv [ArgCount][0] == '-')				// switch character
		{
			switch (toupper (argv [ArgCount][1]))
			{
				case '?' :
				case 'H' : Usage ();
						   break;					// We do not return here
				case 'S' : ArgCount++;
                           if (ArgCount >= argc)
                              exit (0);
						   strcpy (SourceServer, argv [ArgCount]);
                           ServerSpecified = TRUE;
						   break;
				case 'D' : if (TestMode == FALSE)
								TestMode = TRUE;
						   else
								TestMode = FALSE;
						   break;
				case 'L' : if (LoginScript == FALSE)
								LoginScript = TRUE;
						   else
								LoginScript = FALSE;
						   break;
                case 'O' : ArgCount++;
                           if (ArgCount >= argc)
                              exit (0);
                           strcpy (OutputFile, argv [ArgCount]);
                           OutFileSpecified = TRUE;
                           break;
                case 'F' : ArgCount++;
                           if (ArgCount >= argc)
                              exit (0);
                           strcpy (ControlFile, argv [ArgCount]);
                           CtrlFileSpecified = TRUE;
                           break;
                case 'C' : CreateCtrlFile = TRUE;
                           CreateControlFile ();

                default  : printf ("Unknown switch.\n");
                           Usage ();
                           break;
            }
        }
        else
        {
            printf ("Unknown command line option %s", argv [ArgCount]);
            exit (0);
        }
    }
    SetupSourceServer ();
    BrowseBindery ();
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Create the control file for UIMPORT.                                +
//+ Input  : none                                                       +
//+ Output : none                                                       +
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void CreateControlFile (void)
{
    if (!CtrlFileSpecified)
    {
        strcpy (ControlFile, "B2DS.CTL");
        printf ("Name of control file not specified, using B2DS.CTL.\n");
    }
    if ((CFile = fopen(ControlFile, "w")) == NULL)
    {
        printf ("Control file %s could not be created!\n", ControlFile);
        exit (0);
    }

    printf ("Context in which to add users: ");
    gets (Context);
    printf ("\n");
    printf ("Do you wish to use the template user object?");
    Dummy = ' ';
    while ((Dummy != 'Y') && (Dummy != 'N'))
    {
        while (!kbhit ())
        {
            ;
        }
        Dummy = toupper (getche ());
    }
    printf ("\n");
    while (kbhit ())
          getch ();
    printf ("Do you want to create home directories? ");
    CreateHomeDir = ' ';
    while ((CreateHomeDir != 'Y') && (CreateHomeDir != 'N'))
    {
        while (!kbhit ())
        {
            ;
        }
        CreateHomeDir = toupper (getche ());
    }
    printf ("\n");
    while (kbhit ())
          getch ();
    if (CreateHomeDir == 'Y')
    {
        printf ("Please specify a volume where to place homes under: \n");
        gets (HomeDirVolume);

        printf ("Please specify a directory where to place homes under:\n");
        gets (HomeDirBase);
    }

    fprintf (CFile, "Import control\n");
    fprintf (CFile, "\tName context=\"%s\"\n", Context);
    fprintf (CFile, "\tUser template=%c\n", Dummy);
    if (CreateHomeDir == 'Y')
    {
        fprintf (CFile, "\tCreate Home Directory=Y\n");
        fprintf (CFile, "\tHome Directory Path=\"%s\"\n", HomeDirBase);
        fprintf (CFile, "\tHome Directory Volume=\"%s\"\n", HomeDirVolume);
    }
    fprintf (CFile, "Fields\n");
    fprintf (CFile, "\tName\n");            // Login name
    fprintf (CFile, "\tLast Name\n");       // Full name
    fprintf (CFile, "\tHome directory");
    fclose (CFile);
    printf ("Control file has been created!\n");
    if (CreateCtrlFile == TRUE)
       exit (0);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Browse the bindery for user objects.                                +
//+ Input  : none                                                       +
//+ Output : none                                                       +
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void BrowseBindery (void)
{


    if (OutFileSpecified)
       UData = fopen (OutputFile, "w");
    else
    {
       printf ("No output file specified, writing to B2DS.OUT.\n");
       UData = fopen ("B2DS.OUT", "w");
    }
    CreateControlFile ();
    printf ("Writing UIMPORT file, please wait...\n");
    if (UData == NULL)
    {
        printf ("Unable to open output file\n");
        exit (0);
    }
    ObjectID = -1;
    while  (! NWScanObject ((NWCONN_ID) TargetConnectionID,
                              "*",
                              OT_USER,
                              &ObjectID,
                              ObjectName,
                              &ObjectType,
                              &ObjectPropertiesFlag,
                              &ObjectFlags,
                              &ObjectSecurity))
    {
        fprintf (UData, "%s,", ObjectName);
        memset (PropertyValue, '\0', 128);
        if (ObjectPropertiesFlag)       // it has properties
        {
            strcpy ((char *) PropertyValue, "Unknown");  // just in case
            GetPropertyValue (ObjectName, ObjectType, "IDENTIFICATION");
            fprintf (UData, "%s", PropertyValue);

        }
        else
        {
            fprintf (UData, "Unknown");
        }
        if (CreateHomeDir == 'Y')
        {
            fprintf (UData, ",%s:%s/%s", HomeDirVolume,
                                         HomeDirBase,
                                         ObjectName);
        }
        fprintf (UData, "\n");
        if (TestMode == TRUE)
        {
            printf ("User %s - %s.\n", ObjectName, PropertyValue);
        }
    }
    fclose (UData);
}



//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Retrieve the property value for a specific object/property          +
//+ combination.                                                        +
//+ Input  : ObjectName, ObjectType, PropertyName                       +
//+ Output : None                                                       +
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

int GetPropertyValue (char NWFAR *ObjectName,
                      NWOBJ_TYPE ObjectType,
                      char *PropertyName)
{


    NWSEGMENT_NUM DataSetIndex = 1;
    NWFLAGS       MoreFlag;
    NWFLAGS      PropertyFlags;
    NWCCODE      CCode;
    long         SequenceNumber;
    char         FoundProperty [16];
    char         PropertySecurity;
    char         PropertyHasValue;

    SequenceNumber = -1;
    if (! NWScanProperty ((NWCONN_HANDLE) TargetConnectionID,
                             ObjectName,
                             ObjectType,
                             PropertyName,
                             &SequenceNumber,
                             FoundProperty,
                             &PropertyFlags,
                             &PropertySecurity,
                             &PropertyHasValue,
                             &MoreFlag))
    {
        if (PropertyHasValue)
        {
            DataSetIndex = 1;
            CCode = NWReadPropertyValue ((NWCONN_HANDLE) TargetConnectionID,
                                         ObjectName,
                                         ObjectType,
                                         FoundProperty,
                                         DataSetIndex,
                                         PropertyValue,
                                         &MoreFlag,
                                         &PropertyFlags);
            if (CCode != SUCCESSFUL)
            {
               return FALSE;                        // Oops...
            }
            else
            {
                if (MoreFlag != 0x00)
                {
                    printf ("There are more properties of type %s for %s.\n",
                                            PropertyName,
                                            ObjectName);
                }
                return TRUE;
            }
        }
        return FALSE;
    }
    return FALSE;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Set up the source server for retrieving the bindery information.    +
//+ Input  : none                                                       +
//+ Output : none                                                       +
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void SetupSourceServer (void)
{
    if (ServerSpecified == FALSE)
    {
        printf ("No server name specified, using default server.\n");
        CCode = NWGetDefaultConnectionID (&TargetConnectionID);
        if (CCode != SUCCESSFUL)
        {
            printf ("Unable to retrieve default connection: 0x%x", CCode);
            exit (0);
        }
    }
    else
    {
        CCode = NWGetConnectionHandle (strupr (SourceServer),
                                       0,
                                       &TargetConnectionID,
                                       NULL);
        if (CCode != SUCCESSFUL)
        {
            printf ("Unable to retrieve connection for %s: 0x%x",
                                                       SourceServer,
                                                       CCode);
            exit (0);
        }
    }
    CCode = NWGetFileServerName ((NWCONN_HANDLE) TargetConnectionID,
                                                 &ServerName);
    if (CCode != SUCCESSFUL)
    {
        printf ("Error getting server name: 0x%x", CCode);
        exit (0);
    }
    printf ("Gathering users from server %s.\n", ServerName);

}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Display help screen                                                 +
//+ Input  : none                                                       +
//+ Output : none                                                       +
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void Usage (void)
{
    printf ("B2DS will take the bindery of the specified server");
    printf (" or the default server\n");
    printf ("and convert the user objects together with their properties\n");
    printf ("to a 4.0 UIMPORT file.\n");
    printf ("PASSWORD can and will not be converted!\n");
    printf ("Usage: B2DS -<switches>\n");
    printf ("Valid switches are: -? Help\n");
    printf ("                    -H Help\n");
    printf ("                    -S next argument specifies server name\n");
    printf ("                    -C create control file and exit\n");
    printf ("                    -D Test mode, output to screen also\n");
    printf ("                    -F next argument specifies control file\n");
    printf ("->> not implemented -L export login scripts as well\n");
    printf ("                    -O next argument specifies output file\n");
    printf ("Switches can be passed in lower and upper case.\n\n");
    printf ("If no server is specified the default server will be used.\n");
    printf ("In addition to bindery rights be sure that sufficient rights ");
    printf ("are available\n");
    printf ("for accessing the login scripts when -L is used.\n");
    exit (0);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Increment the line number and ask for keypress after screen is full +
//+ Input  : none 														+
//+ Output : none 														+
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void IncLine (void)
{
	LineCount++;
	if (LineCount > 24)
	{
		PressAnyKey ();
		gotoxy (1, wherey ());
#ifndef NLM_VERSION
		clreol ();
#endif
		LineCount = 1;
	}
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+ Give message, wait for key, clear keyboard buffer 					+
//+ Input  : none                                                       +
//+ Output : none                                                       +
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void PressAnyKey (void)
{
	char 	Dummy;

	printf ( "--- Press 'Q' to exit or any other key for more ---");

	while (!kbhit ())
	{
#ifdef NLM_VERSION
		ThreadSwitch ();				// do not monopolize CPU as NLM
#endif
#ifndef NLM_VERSION
		;           					// just do spinwait on DOS
#endif
	}
	while (kbhit ())
	{
		Dummy = toupper (getch ());
		if (Dummy == 'Q')
			exit (0);
	}
}


